@@ -204,18 +204,21 @@ class VQGanVAE(nn.Module):
         image_size,
         channels = 3,
         num_layers = 4,
-        vq_codebook_size = 512,
-        vq_decay = 0.8,
-        vq_commitment_weight = 1.,
+        layer_mults = None,
         l2_recon_loss = False,
         use_hinge_loss = True,
         num_conv_blocks = 1,
         vgg = None,
+        vq_codebook_size = 512,
+        vq_decay = 0.8,
+        vq_commitment_weight = 1.,
+        vq_kmeans_init = True,
         **kwargs
     ):
         super().__init__()
         vq_kwargs, kwargs = groupby_prefix_and_trim('vq_', kwargs)
 
+        self.image_size = image_size
         self.channels = channels
         self.num_layers = num_layers
         self.codebook_size = vq_codebook_size
@@ -223,7 +226,13 @@ class VQGanVAE(nn.Module):
         self.encoders = MList([])
         self.decoders = MList([])
 
-        dims = (dim,) * (num_layers + 1)
+        layer_mults = default(layer_mults, list(map(lambda t: 2 ** t, range(num_layers))))
+        assert len(layer_mults) == num_layers, 'layer multipliers must be equal to designated number of layers'
+
+        layer_dims = [dim * mult for mult in layer_mults]
+        dims = (dim, *layer_dims)
+        codebook_dim = layer_dims[-1]
+
         reversed_dims = tuple(reversed(dims))
         enc_dim_pairs = zip(dims[:-1], dims[1:])
         dec_dim_pairs = zip(reversed_dims[:-1], reversed_dims[1:])
@@ -234,17 +243,18 @@ class VQGanVAE(nn.Module):
 
         for _ in range(num_conv_blocks):
             self.encoders.append(ConvNextBlock(dims[-1]))
-            self.decoders.append(ConvNextBlock(dims[-1]))
+            self.decoders.insert(0, ConvNextBlock(dims[-1]))
 
         self.encoders.insert(0, nn.Conv2d(channels, dim, 3, padding = 1))
         self.decoders.append(nn.Conv2d(dim, channels, 1))
 
         self.vq = VQ(
-            dim = dim,
+            dim = codebook_dim,
             codebook_size = vq_codebook_size,
             decay = vq_decay,
             commitment_weight = vq_commitment_weight,
             accept_image_fmap = True,
+            kmeans_init = vq_kmeans_init,
             **vq_kwargs
         )
 
@@ -294,13 +304,12 @@ class VQGanVAE(nn.Module):
         self,
         img,
         return_loss = False,
-        return_discr_loss = False
+        return_discr_loss = False,
+        return_recons = False
     ):
         batch, channels, device = *img.shape[:2], img.device
         assert channels == self.channels, 'number of channels on image or sketch is not equal to the channels set on this VQGanVAE'
 
-        fmap = img.clone()
-
         fmap, indices, commit_loss = self.encode(img)
 
         fmap = self.decode(fmap)
@@ -316,6 +325,10 @@ class VQGanVAE(nn.Module):
             fmap.detach_()
             fmap_discr_logits, img_discr_logits = map(self.discr, (fmap, img))
             discr_loss = self.discr_loss(fmap_discr_logits, img_discr_logits)
+
+            if return_recons:
+                return discr_loss, fmap
+
             return discr_loss
 
         # perceptual loss
@@ -345,6 +358,10 @@ class VQGanVAE(nn.Module):
         # combine losses
 
         loss = recon_loss + perceptual_loss + commit_loss + adaptive_weight * gen_loss
+
+        if return_recons:
+            return loss, fmap
+
         return loss
 
 # normalizations
